package cn.com.qmth.examcloud.task.api.controller;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.criteria.Predicate;
import javax.transaction.Transactional;

import org.apache.commons.fileupload.disk.DiskFileItem;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import cn.com.qmth.examcloud.commons.base.util.DateUtil;
import cn.com.qmth.examcloud.commons.base.util.DateUtil.DatePatterns;
import cn.com.qmth.examcloud.commons.base.util.PathUtil;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.request.GetExamReq;
import cn.com.qmth.examcloud.task.dao.ExamStudentImportRepo;
import cn.com.qmth.examcloud.task.dao.entity.ExamStudentImportEntity;
import cn.com.qmth.examcloud.task.dao.enums.ExamStudentImportStatus;
import io.swagger.annotations.ApiOperation;

/**
 * 考生导入
 *
 * @author WANGWEI
 * @date 2018年7月31日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.task}" + "examStudentImport")
public class ExamStudentImportController extends ControllerSupport {

	@Value("${$dir}")
	private String dir;

	private static final String EXAM_STUDENT_IMPORT_FILES = "exam_student_import_files";

	@Autowired
	ExamStudentImportRepo examStudentImportRepo;

	@Autowired
	ExamCloudService examCloudService;

	@ApiOperation(value = "导入考试学生", notes = "导入")
	@PostMapping("/import")
	public void importExamStudent(@RequestParam Long examId,
			@RequestParam CommonsMultipartFile file) throws Exception {
		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		GetExamReq req = new GetExamReq();
		req.setId(examId);
		req.setRootOrgId(accessUser.getRootOrgId());
		examCloudService.getExam(req);

		DiskFileItem item = (DiskFileItem) file.getFileItem();
		File storeLocation = item.getStoreLocation();
		String originalFilename = file.getOriginalFilename();

		String suffix = originalFilename.substring(originalFilename.indexOf("."));
		long batchId = System.currentTimeMillis();
		String name = new StringBuilder().append(rootOrgId).append("_").append(examId).append("_")
				.append(DateUtil.getNow(DatePatterns.YYYYMMDDHHMM)).toString();

		String destFileName = name + suffix;
		String resultFileName = name + ".txt";

		String destFilePath = PathUtil
				.getCanonicalPath(dir + "/" + EXAM_STUDENT_IMPORT_FILES + "/" + destFileName);
		File destFile = new File(destFilePath);

		FileUtils.copyFile(storeLocation, destFile);

		ExamStudentImportEntity examStudentImport = new ExamStudentImportEntity();
		examStudentImport.setBatchId(batchId);
		examStudentImport.setFailNum(0L);
		examStudentImport.setSuccessNum(0L);
		examStudentImport.setTotalNum(0L);
		examStudentImport.setFilePath(destFileName);
		examStudentImport.setResultFilePath(resultFileName);
		examStudentImport.setStatus(ExamStudentImportStatus.NONE);
		examStudentImport.setRootOrgId(rootOrgId);
		examStudentImport.setExamId(examId);
		examStudentImport.setFileName(originalFilename);

		examStudentImportRepo.saveAndFlush(examStudentImport);
	}

	@ApiOperation(value = "查询所有用户", notes = "")
	@GetMapping("/all/{curPage}/{pageSize}")
	public Page<ExamStudentImportEntity> getExamStudentImportList(@PathVariable Integer curPage,
			@PathVariable Integer pageSize) {
		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		Specification<ExamStudentImportEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			if (!isSuperAdmin()) {
				predicates.add(cb.equal(root.get("rootOrgId"), rootOrgId));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		Pageable pageable = new PageRequest(curPage, pageSize, Sort.Direction.DESC, "updateTime");
		Page<ExamStudentImportEntity> list = examStudentImportRepo.findAll(specification, pageable);

		return list;
	}

}
